//----------------------------------------------------------------------------
//  Project Fcombo
//  
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    Fcombo Application
//  FILE:         fcombdlg.cpp
//  AUTHOR:       Dr J. Maddock
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TFcomboDlg (TDialog).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include "fcombapp.h"
#include "fcombdlg.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TFcomboDlg, TDialog)
//{{TFcomboDlgRSP_TBL_BEGIN}}
  EV_BN_CLICKED(IDC_CHECKBOX1, BNClicked),
  EV_BN_CLICKED(IDC_CHECKBOX2, RasterClicked),
  EV_BN_CLICKED(IDC_CHECKBOX3, PrinterClicked),
  EV_BN_CLICKED(IDC_CHECKBOX5, FixedClicked),
  EV_WM_MEASUREITEM,
  EV_BN_CLICKED(IDC_BUTTON1, PrinterChange),
  EV_CBN_SELCHANGE(IDC_COMBOBOX1, FontSelchange),
//{{TFcomboDlgRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TFcomboDlg Implementation}}


//--------------------------------------------------------
// TFcomboDlg
// ~~~~~~~~~~
// Construction/Destruction handling.
//
static TFcomboDlgXfer TFcomboDlgData;

TFcomboDlg::TFcomboDlg(TWindow* parent, TResId resId, TModule* module)
:
  TDialog(parent, resId, module)
{
//{{TFcomboDlgXFER_USE}}
  ShowTT = new TCheckBox(this, IDC_CHECKBOX1, 0);
  ShowRaster = new TCheckBox(this, IDC_CHECKBOX2, 0);
  ShowPrinter = new TCheckBox(this, IDC_CHECKBOX3, 0);
  FontCombo = new TFontCombo(this, IDC_COMBOBOX1, 1);
  FixedOnly = new TCheckBox(this, IDC_CHECKBOX5, 0);
  printer = new TPrinter(GetApplication());
  FontCombo->SetPrinter(printer);
  StyleBox = new TFontCombo(this, IDC_COMBOBOX2, 1);
  StyleBox->SetPrinter(printer);

  //SetTransferBuffer(&TFcomboDlgData);
//{{TFcomboDlgXFER_USE_END}}

  // INSERT>> Your constructor code here.
}


TFcomboDlg::~TFcomboDlg()
{
  Destroy();

  // INSERT>> Your destructor code here.
  delete printer;
}

void TFcomboDlg::BNClicked()
{
	ShowTT->Toggle();
	Update();
}


void TFcomboDlg::RasterClicked()
{
	ShowRaster->Toggle();
	Update();
}


void TFcomboDlg::PrinterClicked()
{
	ShowPrinter->Toggle();
	Update();
}


void TFcomboDlg::FixedClicked()
{
	FixedOnly->Toggle();
	Update();
}

void TFcomboDlg::Update()
{
	uint16 flags =
		((ShowTT->GetCheck() == BF_CHECKED) ? FC_TT : 0)
		| ((ShowRaster->GetCheck() == BF_CHECKED) ? FC_RASTER : 0)
		| ((ShowPrinter->GetCheck() == BF_CHECKED) ? FC_PRINTER : 0)
		| ((FixedOnly->GetCheck() == BF_CHECKED) ? FC_FIXED : 0) ;

	FontCombo->SetFlags(flags);
	FontSelchange();
} 

void TFcomboDlg::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.
	ShowTT->SetCheck(BF_CHECKED);
   StyleBox->SetFlags(FC_STYLES);
	ShowRaster->SetCheck(BF_CHECKED);
	ShowPrinter->SetCheck(BF_CHECKED);
	FixedOnly->SetCheck(BF_UNCHECKED);
	StyleBox->SetSelIndex(0);
	Update();
}


void TFcomboDlg::EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
	TWindow* win = ChildWithId(ctrlId);
	if(win)
	{
		win->HandleMessage(WM_MEASUREITEM, ctrlId, (LPARAM)(&measureInfo));
	}
}


void TFcomboDlg::PrinterChange()
{
  // INSERT>> Your code here.
	printer->Setup(this);
	FontCombo->Update();
	FontSelchange();
}


void TFcomboDlg::FontSelchange()
{
	LOGFONT lf;
	FontCombo->GetSelFont(&lf);
	StyleBox->SetFamily(lf.lfFaceName);
}

