//----------------------------------------------------------------------------
//  Project fcombo
//  
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    fcombo.apx Application
//  FILE:         fcombo.cpp
//  AUTHOR:       Dr J. Maddock
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TFontCombo (TComboBox).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>

#include "fcombo.h"


//{{TFontCombo Implementation}}

struct TFontInfo
{
	LOGFONT lf;
	int type;
	char style[60];
	TFontInfo(LOGFONT* f, int type, const char* s);
	TFontInfo(const TFontInfo& i);
	TFontInfo(){}
	bool operator==(const TFontInfo& i)const;
	void operator=(const TFontInfo& i);
};

TFontInfo::TFontInfo(LOGFONT* f, int t, const char* s)
{
	memcpy(&lf, f, sizeof(LOGFONT));
	type = t;
	strcpy(style, s);
}

TFontInfo::TFontInfo(const TFontInfo& i)
{
	memcpy(&lf, &i.lf, sizeof(LOGFONT));
	type = i.type;
	strcpy(style, i.style);
}

void TFontInfo::operator=(const TFontInfo& i)
{
	memcpy(&lf, &i.lf, sizeof(LOGFONT));
	type = i.type;
	strcpy(style, i.style);
}

bool TFontInfo::operator==(const TFontInfo& i)const
{
	return !strcmp(lf.lfFaceName, i.lf.lfFaceName) && !strcmp(style, i.style) && (type == i.type);
}


class TFontList: public TArrayAsVector<TFontInfo>
{
public:
	TFontList(): TArrayAsVector<TFontInfo>(10, 0, 10){}
};


int CALLBACK TFontCombo::MainFontProc(ENUMLOGFONT FAR* lpelf,
					NEWTEXTMETRIC FAR*  lpntm, int  FontType, LPARAM lParam)
{
	TFontCombo* combo = (TFontCombo*)lParam;
	if(!combo)
		return 0;
	int type = 0;
	uint16 flags = combo->GetFlags();
	if((flags & FC_TT) && ((FontType & TRUETYPE_FONTTYPE) == TRUETYPE_FONTTYPE))
	{
		type = TRUETYPE_FONTTYPE;
	}
	else if((flags & FC_PRINTER) && ((FontType & DEVICE_FONTTYPE) == DEVICE_FONTTYPE))
	{
		type = DEVICE_FONTTYPE;
	}
	else if((flags & FC_RASTER) && ((FontType & RASTER_FONTTYPE) == RASTER_FONTTYPE))
	{
		type = RASTER_FONTTYPE;
	}
	bool ok = true;
	//check pitch:
	if((flags & FC_FIXED) && !((lpelf->elfLogFont.lfPitchAndFamily & 3) == FIXED_PITCH))
		ok = false;
	if(type && ok)
	{
		//work out the style:
		char* style;
		char* styles[] = {"Regular", "Bold", "Italic", "Bold Italic"};
		if((FontType & TRUETYPE_FONTTYPE) == TRUETYPE_FONTTYPE)
			style = lpelf->elfStyle;
		else
		{
			//create a style:
			style = (lpelf->elfLogFont.lfWeight > FW_NORMAL) ? //bold
							(lpelf->elfLogFont.lfItalic) ? styles[3] : styles[1] : //not bold
							(lpelf->elfLogFont.lfItalic) ? styles[1] : styles[0];
		}
		combo->AddFont(&lpelf->elfLogFont, type, style);
	}
	return true;
}

TFontCombo::TFontCombo(TWindow* parent, int id, int x, int y, int w, int h, uint32 style, uint textLimit, TModule* module)
		:TComboBox(parent, id, x, y, w, h, style, textLimit, module), Glyphs(module ? module->GetInstance() : ::Module->GetInstance(), "ID_FONTGLYPH")
{
	flags = FC_TT;
	printer = 0;
	fonts = new TFontList();
	TScreenDC dc;
	size = MulDiv(14, dc.GetDeviceCaps(LOGPIXELSY), 72);
	*family = 0;
}

TFontCombo::TFontCombo(TWindow* parent, int ResourceId, uint textLen, TModule* module)
	:TComboBox(parent, ResourceId, textLen, module), Glyphs(module ? module->GetInstance() : ::Module->GetInstance(), "ID_FONTGLYPH")
{
	flags = FC_TT;
	printer = 0;
	fonts = new TFontList();
	TScreenDC dc;
	size = MulDiv(14, dc.GetDeviceCaps(LOGPIXELSY), 72);
	*family = 0;
}

TFontCombo::~TFontCombo()
{
	Destroy(IDCANCEL);
	delete fonts;
}

void TFontCombo::DrawItem(DRAWITEMSTRUCT far& drawInfo)
{
	if((int)(drawInfo.itemID) < 0)
		return;
	TFontInfo& info = fonts->operator[](GetItemData(drawInfo.itemID));
	if(!&info)
		return;

	TColor back = !(drawInfo.itemState & ODS_SELECTED) ? TColor::SysWindow : TColor::SysHighlight;
	TColor text = !(drawInfo.itemState & ODS_SELECTED) ? TColor::SysWindowText : TColor::SysHighlightText;
   bool styleonly = flags & FC_STYLES;

	TDC dc(drawInfo.hDC);
	info.lf.lfHeight = size;
	info.lf.lfWidth = 0;
	TFont font(&info.lf);
	TBrush br(back);
	::FillRect((HDC)dc, &drawInfo.rcItem, (HBRUSH)br);
	dc.SelectObject(font);
	dc.SetTextAlign(TA_TOP | TA_LEFT);
	dc.SetBkColor(back);
	dc.SetTextColor(text);
	dc.TextOut(drawInfo.rcItem.left + (styleonly ? 2 : 20), drawInfo.rcItem.top + 1, styleonly ? info.style : info.lf.lfFaceName);

	if(!styleonly)
	{
		//now draw the glyphs:
		TMemoryDC memDC(Glyphs);
		switch(info.type)
		{
		case TRUETYPE_FONTTYPE:
			dc.SetTextColor(TColor::LtGray);
			dc.BitBlt(2, drawInfo.rcItem.top, 16, 16, memDC, 0, 0, SRCCOPY);
			dc.SetTextColor(text);
			dc.BitBlt(2, drawInfo.rcItem.top, 16, 16, memDC, 16, 0, (drawInfo.itemState & ODS_SELECTED) ? SRCPAINT : SRCAND);
			break;
		case DEVICE_FONTTYPE:
			dc.SetTextColor(text);
			dc.BitBlt(2, drawInfo.rcItem.top, 16, 16, memDC, 48, 0, SRCCOPY);
			break;
		case RASTER_FONTTYPE:
			dc.SetTextColor(TColor::LtRed);
			dc.BitBlt(2, drawInfo.rcItem.top, 16, 16, memDC, 32, 0, SRCCOPY);
			break;
		}
		memDC.RestoreBitmap();
	}
	dc.RestoreObjects();

}

void TFontCombo::MeasureItem (MEASUREITEMSTRUCT far& measureInfo)
{
	measureInfo.itemHeight = size + 2;
	//measureInfo.itemWidth = Width;
}

void TFontCombo::SetupWindow()
{
	TComboBox::SetupWindow();
	Update();
	SetSelIndex(0);
}

void TFontCombo::Update()
{
	if(!IsWindow())
		return;
	//validate family name if appropriate:
	if((flags & FC_STYLES) && !*family)
		strcpy(family, "System");
	if(flags & FC_STYLES)
		flags |= FC_TT | FC_RASTER | FC_PRINTER;
	char temp[256];
	GetText(temp, 255);
	ClearList();
	fonts->Flush();
	TDC* dc;
	if(printer)
	{
		TPrintDialog::TData* Data = printer->GetData();
		dc = new TPrintDC(Data->GetDriverName(), Data->GetDeviceName(),
								 Data->GetOutputName(), Data->GetDevMode());
	}
	else
		dc = new TScreenDC();
	Width = 0;
	::EnumFontFamilies((HDC)*dc, (flags & FC_STYLES) ? family : NULL, (FONTENUMPROC)MainFontProc, (LPARAM)this);
	delete dc;
	if(flags & FC_STYLES)	 //add default styles, synthesised if necessary:
	{
		TFontInfo& i = fonts->operator[](0);
		LOGFONT lf = i.lf;
		int type = i.type;
		lf.lfItalic = false;
		lf.lfWeight = FW_BOLD;
		AddFont(&lf, type, "Bold");
		lf.lfItalic = true;
		lf.lfWeight = FW_NORMAL;
		AddFont(&lf, type, "Italic");
		lf.lfWeight = FW_BOLD;
		AddFont(&lf, type, "Bold Italic");
	}
	SetSelString(temp, 0);
}

void TFontCombo::AddFont(LOGFONT* lf, int type, const char* style)
{
	TFontInfo i(lf, type, style);
	if(fonts->HasMember(i))
		return;         //don't add the same member twice!!

	int index = AddString((flags & FC_STYLES) ? style : lf->lfFaceName);
	if(index < 0)
		return;
	int data = fonts->GetItemsInContainer();
	fonts->AddAt(i, data);
	SetItemData(index, data);
}

void TFontCombo::SetFontSize(int n)
{
	TScreenDC dc;
	size = MulDiv(n, dc.GetDeviceCaps(LOGPIXELSY), 72);
	Invalidate();
}

bool TFontCombo::GetSelFont(LOGFONT* lf)
{
	if(GetSelIndex() < 0) return false;
	TFontInfo& info = fonts->operator[](GetItemData(GetSelIndex()));
	if(!&info)
		return false;
	memcpy(lf, &info.lf, sizeof(LOGFONT));
	return true;
}

void TFontCombo::SetFamily(const char* f)
{
	strcpy(family, f);
	flags |= FC_STYLES;
	Update();
}


